-- I made this while learning about LUA and oUF
-- from the layouts Ammo and P3lim.  Designed around a
-- need for clean and unimposing party / raid frames.
-- Commented for clarity and hopefully bugfree.

-- - Ithilien 2008
------------------------------------------------------

-- the local upvalues bandwagon
local select = select
local UnitClass = UnitClass
local UnitIsDead = UnitIsDead
local UnitIsGhost = UnitIsGhost
local UnitIsPlayer = UnitIsPlayer
local UnitReaction = UnitReaction
local UnitIsConnected = UnitIsConnected
local UnitReactionColor = UnitReactionColor
local RAID_CLASS_COLORS = RAID_CLASS_COLORS
local UnitLevel = UnitLevel
local UnitIsTapped = UnitIsTapped
local UnitIstappedByPlayer = UnitIsTappedByPlayer
local UnitPowerType = UnitPowerType
local UnitClassification = UnitClassification
local UnitCreatureFamily = UnitCreatureFamily
local UnitCreatureType = UnitCreatureType
local UnitCanAttack = UnitCanAttack

-- combopoints for rogue / druid
local playerClass = select(2, UnitClass("player"))
local unfiltered = (playerClass == "ROGUE" or playerClass == "WARRIOR")

-- media for bars
local backdrop = {
	bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
	insets = {left = -1.5, right = -1.5, top = -1.5, bottom = -1.5}	}
local statusbarbg = "Interface\\AddOns\\oUF_Arlyx\\statusbar"
local bordertexture = "Interface\\AddOns\\oUF_Arlyx\\border"
local font, fontHeight = "Interface\\AddOns\\oUF_Arlyx\\font.ttf" , 12

-- rogue color change
local powercolors = oUF.colors.power
powercolors[3].r = 1
powercolors[3].g = 0.85
powercolors[3].b = .1

-- colors to use
local green = { r = 0, g = 1, b = 0 }
local gray = { r = 0.5, g = 0.5, b = 0.5 }
local white = { r = 1, g = 1, b = 1}
local unknown = { r = .41, g = .95, b = .2 }


-- Formats for values in bars
local barFormatPerc		= "%d%%"
local barFormatDeficit	= "|cffff8080%d|r"
local barFormatMin		= "%d"

-- Describe those formats
local function fmt_perc(txt, min, max)
	txt:SetFormattedText(barFormatPerc,floor(min/max*100))
end

local function fmt_deficit(txt, min, max)
	local deficit = min - max
	if deficit < 0 then
		txt:SetFormattedText(barFormatDeficit, deficit)
	else
		txt:SetText("")	 -- show nothing if nothing is missing
	end
end


local function fmt_min(txt, min, max)
	txt:SetText(min) -- just show how much there is now
end

-- Make a db of them  (thx to Ammo for this)
local formats = setmetatable({}, { 
	__index = function(self, key)
		if type(key) == "nil" then return nil end
		if not rawget(self, key) then
			if key:find("raidpet%d") then self[key] = self.raidpet
			elseif key:find("raidtarget%d") then self[key] = self.raidtarget
			elseif key:find("raid%d") then self[key] = self.raid
			elseif key:find("partypet%d") then self[key] = self.partypet
			elseif key:find("party%dtarget") then self[key] = self.partytarget
			elseif key:find("party%d") then self[key] = self.party
			else
				self[key] = {}
			end
		end
		return self[key]
	end,
	__newindex = function(self, key, value)
		rawset(self, key, setmetatable(value, fmtmeta))
	end,
})


-- Now put it to use!
formats.player.health = fmt_min
formats.party.health = fmt_deficit
formats.target.health = fmt_perc
formats.focus.health = fmt_deficit
formats.pet.health = fmt_perc
-- Not in use are:
--formats.target.power
--formats.player.power
--formats.focus.power
--formats.party.power
--formats.raid.power
--formats.raid.health
--formats.partypet.health
--formats.partypet.power
--formats.targettarget.health
--formats.targettarget.power
--formats.focustarget.health
--formats.focustarget.power
--formats.partytarget.power
--formats.partytarget.health

-- Formats for levels
local levelFormats = {
	["worldboss"] = "|cff0090ffBoss|r",
	["rareelite"] = "|cff0090ff%s+ Rare|r",
	["elite"] = "|cff0090ff%s+|r",
	["rare"] = "|cff0090ff%s Rare|r",
	["normal"] = "|cff0090ff%s|r",
	["trivial"] = "|cff0090ffTrivial|r",
}

-- Some colors for target frames based on level difference
local function getDifficultyColor(level)
	local levelDiff = level - UnitLevel("player")
	if levelDiff >= 5 then
		return 1.00, 0.10, 0.10
	elseif levelDiff >= 3 then
		return 1.00, 0.50,0.25
	elseif levelDiff >= -2 then
		return 1.00, 1.00, 1.00
	elseif -levelDiff <= GetQuestGreenRange() then
		return 0.25, 0.75, 0.25
	end
	return 0.50, 0.50, 0.50
end

-- Rightclick menu core
local function menu(self)
	local unit = self.unit:sub(1, -2)
	local cunit = self.unit:gsub("(.)", string.upper, 1)
	if(unit == "party" or unit == "partypet") then
		ToggleDropDownMenu(1, nil, _G["PartyMemberFrame"..self.id.."DropDown"], "cursor", 0, 0)
	elseif(_G[cunit.."FrameDropDown"]) then
		ToggleDropDownMenu(1, nil, _G[cunit.."FrameDropDown"], "cursor", 0, 0)
	end
end

------------------------------------------------------
-------         Update functions ---------------------
--			Format the level text colors
local function updateLevel(self, event, unit)
	if self.unit ~= unit then return end  -- do nothing for targeting self
	
	local lvl = self.Lvl  -- player's level
	local level = UnitLevel(unit)   -- unit's level
	local classification = UnitClassification(unit)  -- type of unit

	if classificationFormats[classification] then
		lvl:SetFormattedText(classificationFormats[classification], level)
	else
		lvl:SetText(level)
	end
	--- for setting colors
	if UnitCanAttack("player", unit) then
		lvl:SetTextColor(getDifficultyColor(level)) -- described above
	else
		lvl:SetTextColor(1, 1, 1) -- make white
	end
end

--			Format the name text
--	Name 00
local updateName = function(self, event, unit)
	if(self.unit ~= unit) then return end

	local name = UnitName(unit)
	local lvl = UnitLevel(unit)

	if lvl < 0 then lvl = "??" end -- slight hack
	if(unit == "target") then
		self.Name:SetFormattedText("%s "..levelFormats[UnitClassification(unit)], name, lvl)
	else
		self.Name:SetText(name)
	end
end

--	--		Format Health Text and Bar
local function updateHealth(self, event, bar, unit, min, max)
	local cur, maxhp, MHfound
	if MobHealth3 then  -- support for MobHealth
		cur,maxhp,MHfound = MobHealth3:GetUnitHealth(unit, min, max)
	end
	if not MHfound then
		cur, maxhp = min, max
	end

	if not unit=="raid" then
		if UnitIsDead(unit) then 
			bar:SetValue(0)
			bar.value:SetText("dead")
		elseif UnitIsGhost(unit) then
			bar:SetValue(0)
			bar.value:SetText("ghost")
		elseif not UnitIsConnected(unit) then
			bar:SetValue(0)
			bar.value:SetText("offline")
		else
			formats[unit].health(bar.value, cur, maxhp)
		end
	end

	local color	-- bar colors
	if unit == "pet" then
		color = oUF.colors.happiness[GetPetHappiness()]
	elseif UnitIsPlayer(unit) then
		color = RAID_CLASS_COLORS[select(2, UnitClass(unit))]
	elseif UnitIsTapped(unit) and not UnitIsTappedByPlayer(unit) and not UnitIsPlayer(unit) then
		color = gray
	else
		color = UnitReactionColor[UnitReaction(unit, "player")]
	end
	if not color then 
		color = unknown
	end
	bar:SetStatusBarColor(color.r, color.g, color.b)
	bar.bg:SetTexture(color.r, color.g, color.b, .25)
end

--	--	--	Format Power Text and Bar
local function updatePower(self, event, bar, unit, min, max)
	if max == 0 or UnitIsDead(unit) or UnitIsGhost(unit) or not UnitIsConnected(unit) then
		bar:SetValue(0)
		if bar.value then
			bar.value:SetText()
		end
	elseif bar.value then
		formats[unit].power(bar.value, min, max)
	end

	local color = powercolors[UnitPowerType(unit)] or gray
	bar:SetStatusBarColor(color.r, color.g, color.b)
	bar.bg:SetTexture(color.r, color.g, color.b, 0.25)	
end


local function noHide(self)
	self:SetVertexColor(.25,.25, .25)
end

--	--	--	Auras and Icons
local function auraIcon(self, button, icons, index, debuff)
	button.cd:SetReverse()
	button.icon:SetTexCoord(.07, .93, .07, .93) --zoom icon, thanks Ammo
	button.overlay:SetTexture(bordertexture)
	button.overlay:SetTexCoord(0,1,0,1)
	button.overlay.Hide = noHide
end

-- Text color highlighting for mouseover, make bright yellow text on mouse-in
local function OnEnter(self)
	self.Name:SetTextColor(1, 1, 0)
	if InCombatLockdown() then return end
	UnitFrame_OnEnter(self)
end

local function OnLeave(self)
	self.Name:SetTextColor(1, 1, 1)
	if InCombatLockdown() then return end
	UnitFrame_OnLeave()
end

-- general font settings

local function getFontString(parent)
	local fs = parent:CreateFontString(nil, "OVERLAY")
	fs:SetFont(font, fontHeight)
	fs:SetShadowColor(0,0,0)
	fs:SetShadowOffset(0.8, -0.8)
	fs:SetTextColor(1,1,1)
	fs:SetJustifyH("LEFT")
	return fs
end


--------------------------------------------------------
--------------------------------------------------------
--		NOW ir MaKE FRAMEZ!! HAHAHAHA!!

local function setStyle(settings, self, unit)
	self.menu = menu -- enabling menus
	self:RegisterForClicks("anyup")
	self:SetAttribute("*type2", "menu")
	self:SetScript("OnEnter", OnEnter)
	self:SetScript("OnLeave", OnLeave)
	
	local slim = settings["slim"]
	local showname = settings["showname"]
	
	self:SetBackdrop(backdrop)	-- set the background
	self:SetBackdropColor(0,0,0,1)
	
	-- some heights of bars
	local hpheight = 20
	local ppheight = 5
	if slim then
		hpheight = 12
	end
	
	--\________________________________________/--
	--==<{ Create the HP bar, add some text }>==--
	local hp = CreateFrame("StatusBar",nil,self)
	hp:SetHeight(hpheight)
	hp:SetStatusBarTexture(statusbarbg)
	if unit=="target" then hp:SetPoint("BOTTOMLEFT") else 
		hp:SetPoint("TOPLEFT")
	end
	if unit=="target" then hp:SetPoint("BOTTOMRIGHT") else hp:SetPoint("TOPRIGHT") end
	-- the background
	hp.bg = hp:CreateTexture(nil, "BORDER")
	hp.bg:SetAllPoints(hp)
	hp.bg:SetTexture(.25,.25,.25) -- not sure why this is....
	-- the text
	hp.value = getFontString(hp)
	hp.value:SetPoint("RIGHT", -2, 0)
	-- lock it in
	self.Health = hp
	self.OverrideUpdateHealth = updateHealth
	-- more text to add
	if showname == true then
		self.Name = getFontString(hp)
		self.Name:SetPoint("LEFT", 2, 0)
		self.UNIT_NAME_UPDATE = updateName
	end
	-- make sure we update level 
	self:RegisterEvent("UNIT_LEVEL")
	self.UNIT_LEVEL = updateName
	
	--\____________________________________________/--
	--==<{ Create the power bar for some frames }>==--
	if not slim then	-- not for raid frames, tot, pet etc..
		local pp = CreateFrame("StatusBar", nil, self)
		pp:SetHeight(ppheight)
		pp:SetStatusBarTexture(statusbarbg)
		pp:SetPoint("LEFT")
		pp:SetPoint("RIGHT")
		if unit == "target" then
			pp:SetPoint("BOTTOM", hp, "TOP", 0, 1)
		else
			pp:SetPoint("TOP", hp, "BOTTOM", 0, -1)
		end
		-- set border
		pp.bg = pp:CreateTexture(nil, "BORDER")
		pp.bg:SetAllPoints(pp)
		pp.bg:SetTexture(.25,.25,.25)
		-- initialize
		self.Power = pp
		self.OverrideUpdatePower = updatePower
	end
	
	--\____________________________________/--
	--==<{ voice support from oUF_Voice }>==--
	local voice = hp:CreateTexture(nil, "OVERLAY")
	voice:SetWidth(18)
	voice:SetHeight(18)
	voice:SetPoint("TOPRIGHT", self, "BOTTOMLEFT", -8 -8)
	voice:Hide()
	self.Voice = voice
	
	--\____________________________________/--
	--==<{ Add some icons to the frames }>==--
	-- raid icons
	local icon = hp:CreateTexture(nil, "OVERLAY")
	icon:SetHeight(16)
	icon:SetWidth(16)
	icon:SetPoint("TOP", self, 0, not realsmall and 8 or 0)
	icon:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcons")
	self.RaidIcon = icon
	-- leader icon
	local leader = hp:CreateTexture(nil, "OVERLAY")
	leader:SetHeight(16)
	leader:SetWidth(16)
	leader:SetPoint("TOPLEFT", self, 0, 8)
	leader:SetTexture("Interface\\GroupFrame\\UI-Group-LeaderIcon")
	self.Leader = leader
	
	--\____________________________________/--
	--==<{ Frame fading for party }>==--
	if not unit then
		self.outsideRangeAlpha = .4
		self.inRangeAlpha = 1
		self.Range = true
	end

	
	return self
	
end

--\________________________/--
--==<{ Establish Styles }>==--

oUF:RegisterStyle("Arlyx", setmetatable({
	["initial-width"] = 199,
	["initial-height"] = 25,
}, {__call = setStyle}))

oUF:RegisterStyle("Arlyx_Party", setmetatable({
	["initial-width"] = 100,
	["initial-height"] = 25,
	["showname"] = true,
}, {__call = setStyle}))

oUF:RegisterStyle("Arlyx_Mini", setmetatable({
	["initial-width"] = 100,
	["initial-height"] = 12,
	["slim"] = true,
}, {__call = setStyle}))

oUF:RegisterStyle("Arlyx_Raid", setmetatable({
	["initial-width"] = 100,
	["initial-height"] = 12,
	["slim"] = true,
}, {__call = setStyle}))

--\________________________/--
--==<{ Big sized frames }>==--
--oUF:SetActiveStyle("Arlyx")

--local player = oUF:Spawn("player", "oUF_Player")
--player:SetPoint("CENTER", UIParent, "CENTER", 0, -265)

--local target = oUF:Spawn("target", "oUF_Target")
--target:SetPoint("CENTER", UIParent, "CENTER", 0, -230)

--\____________________/--
--==<{ Party frames }>==--

oUF:SetActiveStyle("Arlyx_Party")

local party	= oUF:Spawn("header", "oUF_Party")
party:SetPoint("TOPLEFT", UIParent, "LEFT", 8, 0)
party:SetAttribute("yOffset", -10)
party:SetAttribute("showParty", true)
party:Show()

--\__________________________/--
--==<{ Little Health Bars }>==--

--oUF:SetActiveStyle("Arlyx_Mini")

--local pet = oUF:Spawn("pet", "oUF_Pet")
--pet:SetPoint("TOPRIGHT", player, "BOTTOMRIGHT", 0, -5)

--local tot = oUF:Spawn("targettarget", "oUF_TargetTarget")
--tot:SetPoint("TOPLEFT", target, "TOPRIGHT", 10, 0)


--\________________________/--
--==<{ Raid Health Bars }>==--

oUF:SetActiveStyle("Arlyx_Raid")

local raid = oUF:Spawn("header", "oUF_Raid")
raid:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 10, -10)
raid:SetManyAttributes(
	"point", "BOTTOM",
	"sortDir", "DESC",
	"xOffset", 0,
	"yOffset", 4.9,
	"groupFilter", "1,2,3,4,5,6,7,8",
	"groupingOrder", "1,2,3,4,5,6,7,8",
	"groupBy", "GROUP",
	"showRaid", true
)

raid:Show()